<?php

namespace Drupal\commerce_cookie_condition\Plugin\Commerce\Condition;

use Drupal\commerce\Attribute\CommerceCondition;
use Drupal\commerce\Plugin\Commerce\Condition\ConditionBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a condition to check a specific cookie value.
 */
#[CommerceCondition(
  id: "user_cookie_condition",
  label: new TranslatableMarkup("Current user has cookie"),
  entity_type: "commerce_order",
  category: new TranslatableMarkup("Customer"),
  parent_entity_type: "commerce_promotion",
)]
class UserCookieCondition extends ConditionBase implements ContainerFactoryPluginInterface {

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected Request $request;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    RequestStack $request_stack
) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->request = $request_stack->getCurrentRequest();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('request_stack')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
        'cookie_name' => '',
        'cookie_value' => '',
      ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['cookie_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Cookie name'),
      '#description' => $this->t('Name of the cookie to check.'),
      '#default_value' => $this->configuration['cookie_name'],
      '#required' => TRUE,
    ];
    $form['cookie_value'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Expected cookie value'),
      '#description' => $this->t('Expected value of the cookie.'),
      '#default_value' => $this->configuration['cookie_value'],
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    $values = $form_state->getValue($form['#parents']);
    $this->configuration['cookie_name'] = $values['cookie_name'];
    $this->configuration['cookie_value'] = $values['cookie_value'];
  }

  /**
   * {@inheritdoc}
   */
  public function evaluate(EntityInterface $entity) {
    $this->assertEntity($entity);

    $cookie_name = $this->configuration['cookie_name'];
    $expected_value = $this->configuration['cookie_value'];
    $actual_value = $this->request->cookies->get($cookie_name);

    return $this->request->cookies->has($cookie_name) && $actual_value == $expected_value;
  }

  /**
   * {@inheritdoc}
   */
  public function summary() {
    return $this->t('The user has a cookie "@name" with value "@value".', [
      '@name' => $this->configuration['cookie_name'],
      '@value' => $this->configuration['cookie_value'],
    ]);
  }

}
